@@ -17,16 +17,25 @@ import collections
 
 
 from octavia.common import constants
+from octavia.tests.unit.common.sample_configs import sample_certs
 
 
 def sample_amphora_tuple():
-    amphora = collections.namedtuple('amphora', 'id, load_balancer_id, '
-                                                'compute_id, status,'
-                                                'lb_network_ip, vrrp_ip')
-    return amphora(id='sample_amp_id_1', load_balancer_id='sample_lb_id_1',
-                   compute_id='sample_compute_id_1', status='ACTIVE',
-                   lb_network_ip='10.0.0.1',
-                   vrrp_ip='10.0.0.2')
+    in_amphora = collections.namedtuple(
+        'amphora', 'id, lb_network_ip, vrrp_ip, ha_ip, vrrp_port_id, '
+                   'ha_port_id, role, status, vrrp_interface,'
+                   'vrrp_priority')
+    return in_amphora(
+        id='sample_amphora_id_1',
+        lb_network_ip='10.0.1.1',
+        vrrp_ip='10.1.1.1',
+        ha_ip='192.168.10.1',
+        vrrp_port_id='1234',
+        ha_port_id='1234',
+        role=None,
+        status='ACTIVE',
+        vrrp_interface=None,
+        vrrp_priority=None)
 
 RET_PERSISTENCE = {
     'type': 'HTTP_COOKIE',
@@ -63,7 +72,9 @@ RET_MEMBER_1 = {
     'weight': 13,
     'subnet_id': '10.0.0.1/24',
     'enabled': True,
-    'operating_status': 'ACTIVE'}
+    'operating_status': 'ACTIVE',
+    'monitor_address': None,
+    'monitor_port': None}
 
 RET_MEMBER_2 = {
     'id': 'sample_member_id_2',
@@ -72,7 +83,9 @@ RET_MEMBER_2 = {
     'weight': 13,
     'subnet_id': '10.0.0.1/24',
     'enabled': True,
-    'operating_status': 'ACTIVE'}
+    'operating_status': 'ACTIVE',
+    'monitor_address': None,
+    'monitor_port': None}
 
 RET_MEMBER_3 = {
     'id': 'sample_member_id_3',
@@ -81,7 +94,9 @@ RET_MEMBER_3 = {
     'weight': 13,
     'subnet_id': '10.0.0.1/24',
     'enabled': True,
-    'operating_status': 'ACTIVE'}
+    'operating_status': 'ACTIVE',
+    'monitor_address': None,
+    'monitor_port': None}
 
 RET_POOL_1 = {
     'id': 'sample_pool_id_1',
@@ -118,7 +133,8 @@ RET_L7RULE_1 = {
     'compare_type': constants.L7RULE_COMPARE_TYPE_STARTS_WITH,
     'key': None,
     'value': '/api',
-    'invert': False}
+    'invert': False,
+    'enabled': True}
 
 RET_L7RULE_2 = {
     'id': 'sample_l7rule_id_2',
@@ -126,7 +142,8 @@ RET_L7RULE_2 = {
     'compare_type': constants.L7RULE_COMPARE_TYPE_CONTAINS,
     'key': 'Some-header',
     'value': 'This\\ string\\\\\\ with\\ stuff',
-    'invert': True}
+    'invert': True,
+    'enabled': True}
 
 RET_L7RULE_3 = {
     'id': 'sample_l7rule_id_3',
@@ -134,7 +151,8 @@ RET_L7RULE_3 = {
     'compare_type': constants.L7RULE_COMPARE_TYPE_REGEX,
     'key': 'some-cookie',
     'value': 'this.*|that',
-    'invert': False}
+    'invert': False,
+    'enabled': True}
 
 RET_L7RULE_4 = {
     'id': 'sample_l7rule_id_4',
@@ -142,7 +160,8 @@ RET_L7RULE_4 = {
     'compare_type': constants.L7RULE_COMPARE_TYPE_EQUAL_TO,
     'key': None,
     'value': 'jpg',
-    'invert': False}
+    'invert': False,
+    'enabled': True}
 
 RET_L7RULE_5 = {
     'id': 'sample_l7rule_id_5',
@@ -150,7 +169,17 @@ RET_L7RULE_5 = {
     'compare_type': constants.L7RULE_COMPARE_TYPE_ENDS_WITH,
     'key': None,
     'value': '.example.com',
-    'invert': False}
+    'invert': False,
+    'enabled': True}
+
+RET_L7RULE_6 = {
+    'id': 'sample_l7rule_id_6',
+    'type': constants.L7RULE_TYPE_HOST_NAME,
+    'compare_type': constants.L7RULE_COMPARE_TYPE_ENDS_WITH,
+    'key': None,
+    'value': '.example.com',
+    'invert': False,
+    'enabled': False}
 
 RET_L7POLICY_1 = {
     'id': 'sample_l7policy_id_1',
@@ -192,6 +221,14 @@ RET_L7POLICY_5 = {
     'enabled': False,
     'l7rules': [RET_L7RULE_5]}
 
+RET_L7POLICY_6 = {
+    'id': 'sample_l7policy_id_6',
+    'action': constants.L7POLICY_ACTION_REJECT,
+    'redirect_pool': None,
+    'redirect_url': None,
+    'enabled': True,
+    'l7rules': []}
+
 RET_LISTENER = {
     'id': 'sample_listener_id_1',
     'protocol_port': '80',
@@ -204,7 +241,8 @@ RET_LISTENER = {
     'topology': 'SINGLE',
     'pools': [RET_POOL_1],
     'l7policies': [],
-    'enabled': True}
+    'enabled': True,
+    'insert_headers': {}}
 
 RET_LISTENER_L7 = {
     'id': 'sample_listener_id_1',
@@ -218,8 +256,9 @@ RET_LISTENER_L7 = {
     'topology': 'SINGLE',
     'pools': [RET_POOL_1, RET_POOL_2],
     'l7policies': [RET_L7POLICY_1, RET_L7POLICY_2, RET_L7POLICY_3,
-                   RET_L7POLICY_4, RET_L7POLICY_5],
-    'enabled': True}
+                   RET_L7POLICY_4, RET_L7POLICY_5, RET_L7POLICY_6],
+    'enabled': True,
+    'insert_headers': {}}
 
 RET_LISTENER_TLS = {
     'id': 'sample_listener_id_1',
@@ -233,7 +272,8 @@ RET_LISTENER_TLS = {
     'default_tls_container': RET_DEF_TLS_CONT,
     'pools': [RET_POOL_1],
     'l7policies': [],
-    'enabled': True}
+    'enabled': True,
+    'insert_headers': {}}
 
 RET_LISTENER_TLS_SNI = {
     'id': 'sample_listener_id_1',
@@ -250,33 +290,52 @@ RET_LISTENER_TLS_SNI = {
     'sni_containers': [RET_SNI_CONT_1, RET_SNI_CONT_2],
     'pools': [RET_POOL_1],
     'l7policies': [],
-    'enabled': True}
+    'enabled': True,
+    'insert_headers': {}}
+
+RET_AMPHORA = {
+    'id': 'sample_amphora_id_1',
+    'lb_network_ip': '10.0.1.1',
+    'vrrp_ip': '10.1.1.1',
+    'ha_ip': '192.168.10.1',
+    'vrrp_port_id': '1234',
+    'ha_port_id': '1234',
+    'role': None,
+    'status': 'ACTIVE',
+    'vrrp_interface': None,
+    'vrrp_priority': None}
 
 RET_LB = {
+    'host_amphora': RET_AMPHORA,
     'name': 'test-lb',
     'vip_address': '10.0.0.2',
     'listener': RET_LISTENER,
     'topology': 'SINGLE',
-    'enabled': True}
+    'enabled': True,
+    'global_connection_limit': 98}
 
 RET_LB_TLS = {
     'name': 'test-lb',
     'vip_address': '10.0.0.2',
     'listener': RET_LISTENER_TLS,
-    'enabled': True}
+    'enabled': True,
+    'global_connection_limit': 98}
 
 RET_LB_TLS_SNI = {
     'name': 'test-lb',
     'vip_address': '10.0.0.2',
     'listener': RET_LISTENER_TLS_SNI,
-    'enabled': True}
+    'enabled': True,
+    'global_connection_limit': 98}
 
 RET_LB_L7 = {
+    'host_amphora': RET_AMPHORA,
     'name': 'test-lb',
     'vip_address': '10.0.0.2',
     'listener': RET_LISTENER_L7,
     'topology': 'SINGLE',
-    'enabled': True}
+    'enabled': True,
+    'global_connection_limit': 98}
 
 
 def sample_loadbalancer_tuple(proto=None, monitor=True, persistence=True,
@@ -345,40 +404,48 @@ def sample_vip_tuple():
 def sample_listener_tuple(proto=None, monitor=True, persistence=True,
                           persistence_type=None, persistence_cookie=None,
                           tls=False, sni=False, peer_port=None, topology=None,
-                          l7=False, enabled=True):
+                          l7=False, enabled=True, insert_headers=None,
+                          be_proto=None, monitor_ip_port=False,
+                          monitor_proto=None):
     proto = 'HTTP' if proto is None else proto
-    be_proto = 'HTTP' if proto is 'TERMINATED_HTTPS' else proto
+    if be_proto is None:
+        be_proto = 'HTTP' if proto is 'TERMINATED_HTTPS' else proto
     topology = 'SINGLE' if topology is None else topology
     port = '443' if proto is 'HTTPS' or proto is 'TERMINATED_HTTPS' else '80'
     peer_port = 1024 if peer_port is None else peer_port
+    insert_headers = insert_headers or {}
     in_listener = collections.namedtuple(
         'listener', 'id, project_id, protocol_port, protocol, default_pool, '
                     'connection_limit, tls_certificate_id, '
                     'sni_container_ids, default_tls_container, '
                     'sni_containers, load_balancer, peer_port, pools, '
-                    'l7policies, enabled',)
+                    'l7policies, enabled, insert_headers',)
     if l7:
         pools = [
             sample_pool_tuple(
                 proto=be_proto, monitor=monitor, persistence=persistence,
                 persistence_type=persistence_type,
-                persistence_cookie=persistence_cookie),
+                persistence_cookie=persistence_cookie,
+                monitor_ip_port=monitor_ip_port, monitor_proto=monitor_proto),
             sample_pool_tuple(
                 proto=be_proto, monitor=monitor, persistence=persistence,
                 persistence_type=persistence_type,
-                persistence_cookie=persistence_cookie, sample_pool=2)]
+                persistence_cookie=persistence_cookie, sample_pool=2,
+                monitor_ip_port=monitor_ip_port, monitor_proto=monitor_proto)]
         l7policies = [
             sample_l7policy_tuple('sample_l7policy_id_1', sample_policy=1),
             sample_l7policy_tuple('sample_l7policy_id_2', sample_policy=2),
             sample_l7policy_tuple('sample_l7policy_id_3', sample_policy=3),
             sample_l7policy_tuple('sample_l7policy_id_4', sample_policy=4),
-            sample_l7policy_tuple('sample_l7policy_id_5', sample_policy=5)]
+            sample_l7policy_tuple('sample_l7policy_id_5', sample_policy=5),
+            sample_l7policy_tuple('sample_l7policy_id_6', sample_policy=6)]
     else:
         pools = [
             sample_pool_tuple(
                 proto=be_proto, monitor=monitor, persistence=persistence,
                 persistence_type=persistence_type,
-                persistence_cookie=persistence_cookie)]
+                persistence_cookie=persistence_cookie,
+                monitor_ip_port=monitor_ip_port, monitor_proto=monitor_proto)]
         l7policies = []
     return in_listener(
         id='sample_listener_id_1',
@@ -391,44 +458,50 @@ def sample_listener_tuple(proto=None, monitor=True, persistence=True,
         default_pool=sample_pool_tuple(
             proto=be_proto, monitor=monitor, persistence=persistence,
             persistence_type=persistence_type,
-            persistence_cookie=persistence_cookie),
+            persistence_cookie=persistence_cookie,
+            monitor_ip_port=monitor_ip_port, monitor_proto=monitor_proto),
         connection_limit=98,
         tls_certificate_id='cont_id_1' if tls else '',
         sni_container_ids=['cont_id_2', 'cont_id_3'] if sni else [],
         default_tls_container=sample_tls_container_tuple(
-            id='cont_id_1', certificate='--imapem1--\n',
-            private_key='--imakey1--\n', intermediates=[
-                '--imainter1--\n', '--imainter1too--\n'],
-            primary_cn='aFakeCN'
+            id='cont_id_1', certificate=sample_certs.X509_CERT,
+            private_key=sample_certs.X509_CERT_KEY,
+            intermediates=sample_certs.X509_IMDS_LIST,
+            primary_cn=sample_certs.X509_CERT_CN
         ) if tls else '',
         sni_containers=[
             sample_tls_sni_container_tuple(
+                tls_container_id='cont_id_2',
                 tls_container=sample_tls_container_tuple(
-                    id='cont_id_2', certificate='--imapem2--\n',
-                    private_key='--imakey2--\n', intermediates=[
-                        '--imainter2--\n', '--imainter2too--\n'
-                    ], primary_cn='aFakeCN')),
+                    id='cont_id_2', certificate=sample_certs.X509_CERT_2,
+                    private_key=sample_certs.X509_CERT_KEY_2,
+                    intermediates=sample_certs.X509_IMDS_LIST,
+                    primary_cn=sample_certs.X509_CERT_CN_2)),
             sample_tls_sni_container_tuple(
+                tls_container_id='cont_id_3',
                 tls_container=sample_tls_container_tuple(
-                    id='cont_id_3', certificate='--imapem3--\n',
-                    private_key='--imakey3--\n', intermediates=[
-                        '--imainter3--\n', '--imainter3too--\n'
-                    ], primary_cn='aFakeCN'))]
+                    id='cont_id_3', certificate=sample_certs.X509_CERT_3,
+                    private_key=sample_certs.X509_CERT_KEY_3,
+                    intermediates=sample_certs.X509_IMDS_LIST,
+                    primary_cn=sample_certs.X509_CERT_CN_3))]
         if sni else [],
         pools=pools,
         l7policies=l7policies,
-        enabled=enabled
+        enabled=enabled,
+        insert_headers=insert_headers
     )
 
 
-def sample_tls_sni_container_tuple(tls_container=None):
-    sc = collections.namedtuple('sni_container', 'tls_container')
-    return sc(tls_container=tls_container)
+def sample_tls_sni_container_tuple(tls_container_id=None, tls_container=None):
+    sc = collections.namedtuple('sni_container', 'tls_container_id, '
+                                                 'tls_container')
+    return sc(tls_container_id=tls_container_id, tls_container=tls_container)
 
 
-def sample_tls_sni_containers_tuple(tls_container=None):
-    sc = collections.namedtuple('sni_containers', 'tls_container')
-    return [sc(tls_container=tls_container)]
+def sample_tls_sni_containers_tuple(tls_container_id=None, tls_container=None):
+    sc = collections.namedtuple('sni_containers', 'tls_container_id, '
+                                                  'tls_container')
+    return [sc(tls_container_id=tls_container_id, tls_container=tls_container)]
 
 
 def sample_tls_container_tuple(id='cont_id_1', certificate=None,
@@ -443,8 +516,10 @@ def sample_tls_container_tuple(id='cont_id_1', certificate=None,
 
 def sample_pool_tuple(proto=None, monitor=True, persistence=True,
                       persistence_type=None, persistence_cookie=None,
-                      sample_pool=1):
+                      sample_pool=1, monitor_ip_port=False,
+                      monitor_proto=None):
     proto = 'HTTP' if proto is None else proto
+    monitor_proto = proto if monitor_proto is None else monitor_proto
     in_pool = collections.namedtuple(
         'pool', 'id, protocol, lb_algorithm, members, health_monitor,'
                 'session_persistence, enabled, operating_status')
@@ -454,15 +529,18 @@ def sample_pool_tuple(proto=None, monitor=True, persistence=True,
     mon = None
     if sample_pool == 1:
         id = 'sample_pool_id_1'
-        members = [sample_member_tuple('sample_member_id_1', '10.0.0.99'),
-                   sample_member_tuple('sample_member_id_2', '10.0.0.98')]
+        members = [sample_member_tuple('sample_member_id_1', '10.0.0.99',
+                                       monitor_ip_port=monitor_ip_port),
+                   sample_member_tuple('sample_member_id_2', '10.0.0.98',
+                                       monitor_ip_port=monitor_ip_port)]
         if monitor is True:
-            mon = sample_health_monitor_tuple(proto=proto)
+            mon = sample_health_monitor_tuple(proto=monitor_proto)
     elif sample_pool == 2:
         id = 'sample_pool_id_2'
-        members = [sample_member_tuple('sample_member_id_3', '10.0.0.97')]
+        members = [sample_member_tuple('sample_member_id_3', '10.0.0.97',
+                                       monitor_ip_port=monitor_ip_port)]
         if monitor is True:
-            mon = sample_health_monitor_tuple(proto=proto, sample_hm=2)
+            mon = sample_health_monitor_tuple(proto=monitor_proto, sample_hm=2)
     return in_pool(
         id=id,
         protocol=proto,
@@ -474,11 +552,15 @@ def sample_pool_tuple(proto=None, monitor=True, persistence=True,
         operating_status='ACTIVE')
 
 
-def sample_member_tuple(id, ip, enabled=True, operating_status='ACTIVE'):
+def sample_member_tuple(id, ip, enabled=True, operating_status='ACTIVE',
+                        monitor_ip_port=False):
     in_member = collections.namedtuple('member',
                                        'id, ip_address, protocol_port, '
                                        'weight, subnet_id, '
-                                       'enabled, operating_status')
+                                       'enabled, operating_status, '
+                                       'monitor_address, monitor_port')
+    monitor_address = '192.168.1.1' if monitor_ip_port else None
+    monitor_port = 9000 if monitor_ip_port else None
     return in_member(
         id=id,
         ip_address=ip,
@@ -486,7 +568,9 @@ def sample_member_tuple(id, ip, enabled=True, operating_status='ACTIVE'):
         weight=13,
         subnet_id='10.0.0.1/24',
         enabled=enabled,
-        operating_status=operating_status)
+        operating_status=operating_status,
+        monitor_address=monitor_address,
+        monitor_port=monitor_port)
 
 
 def sample_session_persistence_tuple(persistence_type=None,
@@ -523,6 +607,7 @@ def sample_l7policy_tuple(id,
     in_l7policy = collections.namedtuple('l7policy',
                                          'id, action, redirect_pool, '
                                          'redirect_url, l7rules, enabled')
+    l7rules = []
     if sample_policy == 1:
         action = constants.L7POLICY_ACTION_REDIRECT_TO_POOL
         redirect_pool = sample_pool_tuple(sample_pool=2)
@@ -538,11 +623,13 @@ def sample_l7policy_tuple(id,
                    sample_l7rule_tuple('sample_l7rule_id_5', sample_rule=5)]
     elif sample_policy == 4:
         action = constants.L7POLICY_ACTION_REJECT
-        l7rules = []
     elif sample_policy == 5:
         action = constants.L7POLICY_ACTION_REJECT
         enabled = False
         l7rules = [sample_l7rule_tuple('sample_l7rule_id_5', sample_rule=5)]
+    elif sample_policy == 6:
+        action = constants.L7POLICY_ACTION_REJECT
+        l7rules = [sample_l7rule_tuple('sample_l7rule_id_6', sample_rule=6)]
     return in_l7policy(
         id=id,
         action=action,
@@ -558,47 +645,60 @@ def sample_l7rule_tuple(id,
                         key=None,
                         value='/api',
                         invert=False,
+                        enabled=True,
                         sample_rule=1):
     in_l7rule = collections.namedtuple('l7rule',
                                        'id, type, compare_type, '
-                                       'key, value, invert')
+                                       'key, value, invert, enabled')
     if sample_rule == 2:
         type = constants.L7RULE_TYPE_HEADER
         compare_type = constants.L7RULE_COMPARE_TYPE_CONTAINS
         key = 'Some-header'
         value = 'This string\\ with stuff'
         invert = True
+        enabled = True
     if sample_rule == 3:
         type = constants.L7RULE_TYPE_COOKIE
         compare_type = constants.L7RULE_COMPARE_TYPE_REGEX
         key = 'some-cookie'
         value = 'this.*|that'
         invert = False
+        enabled = True
     if sample_rule == 4:
         type = constants.L7RULE_TYPE_FILE_TYPE
         compare_type = constants.L7RULE_COMPARE_TYPE_EQUAL_TO
         key = None
         value = 'jpg'
         invert = False
+        enabled = True
     if sample_rule == 5:
         type = constants.L7RULE_TYPE_HOST_NAME
         compare_type = constants.L7RULE_COMPARE_TYPE_ENDS_WITH
         key = None
         value = '.example.com'
         invert = False
+        enabled = True
+    if sample_rule == 6:
+        type = constants.L7RULE_TYPE_HOST_NAME
+        compare_type = constants.L7RULE_COMPARE_TYPE_ENDS_WITH
+        key = None
+        value = '.example.com'
+        invert = False
+        enabled = False
     return in_l7rule(
         id=id,
         type=type,
         compare_type=compare_type,
         key=key,
         value=value,
-        invert=invert)
+        invert=invert,
+        enabled=enabled)
 
 
 def sample_base_expected_config(frontend=None, backend=None, peers=None):
     if frontend is None:
         frontend = ("frontend sample_listener_id_1\n"
-                    "    option tcplog\n"
+                    "    option httplog\n"
                     "    maxconn 98\n"
                     "    bind 10.0.0.2:80\n"
                     "    mode http\n"
@@ -611,7 +711,7 @@ def sample_base_expected_config(frontend=None, backend=None, peers=None):
                    "    timeout check 31\n"
                    "    option httpchk GET /index.html\n"
                    "    http-check expect rstatus 418\n"
-                   "    option forwardfor\n"
+                   "    fullconn 98\n"
                    "    server sample_member_id_1 10.0.0.99:82 weight 13 "
                    "check inter 30s fall 3 rise 2 cookie sample_member_id_1\n"
                    "    server sample_member_id_2 10.0.0.98:82 weight 13 "
@@ -626,7 +726,8 @@ def sample_base_expected_config(frontend=None, backend=None, peers=None):
             "    log /dev/log local0\n"
             "    log /dev/log local1 notice\n"
             "    stats socket /var/lib/octavia/sample_listener_id_1.sock"
-            " mode 0666 level user\n\n"
+            " mode 0666 level user\n"
+            "    maxconn 98\n\n"
             "defaults\n"
             "    log global\n"
             "    retries 3\n"
